#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>

#include <linux/videodev2.h>

#include <SDL.h>
#include <SDL_image.h>

#define SCREEN_WIDTH  (640)
#define SCREEN_HEIGHT (480)

#define V4L2_CID_PAN_RELATIVE_LOGITECH  0x0A046D01
#define V4L2_CID_TILT_RELATIVE_LOGITECH 0x0A046D02
#define V4L2_CID_PANTILT_RESET_LOGITECH 0x0A046D03
#define ONE_DEGREE (64)
#define ONE_STEP (ONE_DEGREE * 3)

static void
die(char *message)
{
  perror(message);
  exit(1);
}

static void
pantilt_reset(int fd)
{
  struct v4l2_ext_control xctrls[1];
  struct v4l2_ext_controls ctrls;
  int ret;

  ctrls.count = 1;
  ctrls.controls = xctrls;
  xctrls[0].id = V4L2_CID_PANTILT_RESET_LOGITECH;
  xctrls[0].value = 3;
  ret = ioctl(fd, VIDIOC_S_EXT_CTRLS, &ctrls);
  if (ret < 0)
    die("pan tilt reset");
  sleep(4);
  return;
}

static void
handle_keyup(SDL_KeyboardEvent *ev,
	     int *done, int fd)
{
  SDL_keysym *keysym;
  SDL_Event quit_ev;
  struct v4l2_ext_control xctrls[2];
  struct v4l2_ext_controls ctrls;
  int ret;

  keysym = &(ev->keysym);
  if (keysym->sym == SDLK_ESCAPE) {
    memset(&quit_ev, 0, sizeof(quit_ev));
    quit_ev.type = SDL_QUIT;
    SDL_PushEvent(&quit_ev);
    return;
  }
  ctrls.count = 2;
  ctrls.controls = xctrls;
  xctrls[0].id = V4L2_CID_PAN_RELATIVE_LOGITECH;
  xctrls[1].id = V4L2_CID_TILT_RELATIVE_LOGITECH;
  switch (keysym->sym) {
  case SDLK_UP:
    xctrls[0].value = 0;
    xctrls[1].value = -ONE_STEP;
    printf("up\n");
    break;
  case SDLK_DOWN:
    xctrls[0].value = 0;
    xctrls[1].value = ONE_STEP;
    printf("down\n");
    break;
  case SDLK_RIGHT:
    xctrls[0].value = -ONE_STEP;
    xctrls[1].value = 0;
    printf("right\n");
    break;
  case SDLK_LEFT:
    xctrls[0].value = ONE_STEP;
    xctrls[1].value = 0;
    printf("left\n");
    break;
  default:
    /* not interested in other keys. return now */
    return;
    break;
  }
  ret = ioctl(fd, VIDIOC_S_EXT_CTRLS, &ctrls);
  if (ret < 0)
    die("VIDIOC_S_EXT_CTRLS");
}

int main(int argc, char *argv[])
{
  int fd;
  SDL_Surface *s;
  SDL_Surface *image;
  int ret;
  int done = 0;

  ret = SDL_Init(SDL_INIT_VIDEO);
  if (ret < 0)
    die("sdl init");
  atexit(SDL_Quit);

  fd = open("/dev/video0", O_RDONLY);
  if (fd < 0)
    die("open");
  pantilt_reset(fd);

  image = IMG_Load("sapporo.jpg");
  if (!image)
    die("failed to load image\n");

  s = SDL_SetVideoMode(SCREEN_WIDTH,
		       SCREEN_HEIGHT,
		       24, SDL_FULLSCREEN);
  if (!s)
    die("failed to set mode\n");

  SDL_BlitSurface(image, NULL, s, NULL);
  SDL_Flip(s);

  while (!done) {
    int ok;
    SDL_Event ev;

    ok = SDL_WaitEvent(&ev);
    if (!ok)
      break;
    switch (ev.type) {
    case SDL_KEYUP:
      handle_keyup((SDL_KeyboardEvent *)&ev,
		   &done, fd);
      break;
    case SDL_QUIT:
      printf("quiting...\n");
      done = 1;
      break;
    default:
      printf("ev %d\n", ev.type);
      break;
    }
  }

  return 0;
}
